"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const config_1 = __importDefault(require("../config"));
const File = __importStar(require("../file"));
const utils_1 = require("../templates/utils");
const add_1 = __importDefault(require("../templates/add"));
const _common_1 = require("./_common");
const bold = (str) => '\x1b[1m' + str + '\x1b[0m';
const cyan = (str) => '\x1b[36m' + str + '\x1b[0m';
const green = (str) => '\x1b[32m' + str + '\x1b[0m';
const yellow = (str) => '\x1b[33m' + str + '\x1b[0m';
const pink = (str) => '\x1b[35m' + str + '\x1b[0m';
// Scaffold a new elm-spa page
exports.default = {
    run: async () => {
        let [url, template] = process.argv.slice(3);
        if (!url) {
            return Promise.reject(example);
        }
        const page = utils_1.urlArgumentToPages(url);
        const outputFilepath = path_1.default.join(config_1.default.folders.pages.src, ...page) + '.elm';
        let contents = add_1.default(page);
        if (template) {
            const availableTemplates = await _common_1.createMissingAddTemplates();
            const templateSrc = path_1.default.join(config_1.default.folders.templates.user, template + '.elm');
            contents = await File.read(templateSrc).catch(_ => Promise.reject(template404(url, template, availableTemplates)));
            contents = contents.split('{{module}}').join(page.join('.'));
        }
        await File.create(outputFilepath, contents);
        return `  ${bold('New page created at:')}\n  ${outputFilepath}`;
    }
};
const example = '  ' + `
  ${bold(`elm-spa add`)} <url> [template]

  Examples:
  ${bold(`elm-spa ${cyan(`add`)}`)} ${yellow('/')} . . . . . . . . adds a homepage
  ${bold(`elm-spa ${cyan(`add`)}`)} ${yellow('/about-us')} . . . . adds a static route
  ${bold(`elm-spa ${cyan(`add`)}`)} ${yellow('/people/:id')} . . . adds a dynamic route

  Examples with templates:
  ${bold(`elm-spa ${cyan(`add`)}`)} ${yellow('/')} ${pink('static')}
  ${bold(`elm-spa ${cyan(`add`)}`)} ${yellow('/about-us')} ${pink('sandbox')}
  ${bold(`elm-spa ${cyan(`add`)}`)} ${yellow('/people/:id')} ${pink('element')}

  Visit ${green(`https://elm-spa.dev/guide/cli`)} for more details!
`.trim();
const template404 = (url, template, suggestions) => {
    const suggest = `
  Here are the available templates:

    ${suggestions.map(temp => `${yellow(`elm-spa add`)} ${yellow(url)} ${bold(pink(temp))}`).join('\n    ')}
  `;
    return '  ' + `
  ${bold(`elm-spa`)} couldn't find a ${bold(pink(template))} template
  in the ${cyan('.elm-spa/templates')} folder.
  ${suggestions.length ? suggest : ''}
  Visit ${green(`https://elm-spa.dev/guide/cli`)} for more details!

`.trim();
};
