"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chokidar_1 = __importDefault(require("chokidar"));
const url_1 = __importDefault(require("url"));
const http_1 = __importDefault(require("http"));
const websocket_1 = __importDefault(require("websocket"));
const path_1 = __importDefault(require("path"));
const File = __importStar(require("../file"));
const watch_1 = require("./watch");
const terminal_1 = require("../terminal");
const mime_1 = __importDefault(require("mime"));
const fs_1 = require("fs");
const start = async () => new Promise((resolve, reject) => {
    const config = {
        port: process.env.PORT || 1234,
        base: path_1.default.join(process.cwd(), 'public'),
        index: 'index.html'
    };
    const contentType = (extension) => mime_1.default.getType(extension) || 'text/plain';
    const server = http_1.default.createServer(async (req, res) => {
        const url = url_1.default.parse(req.url || '');
        const error = () => {
            res.statusCode = 404;
            res.setHeader('Content-Type', 'text/plain');
            res.write('File not found.');
            res.end();
        };
        if ((url.pathname || '').includes('.')) {
            try {
                const filepath = path_1.default.join(config.base, ...(url.pathname || '').split('/'));
                const s = fs_1.createReadStream(filepath);
                s.on('open', () => {
                    const extension = (url.pathname || '').split('.').slice(-1)[0];
                    res.setHeader('Content-Type', contentType(extension));
                    s.pipe(res);
                });
                s.on('error', error);
            }
            catch (_) {
                error();
            }
        }
        else {
            let file = await File.read(path_1.default.join(config.base, config.index));
            file = file.split('</body>').join(`  <script>${script}</script>\n</body>`);
            res.setHeader('Content-Type', contentType('html'));
            res.write(file);
            res.end();
        }
    });
    // Websockets for live-reloading
    const connections = {};
    const ws = new websocket_1.default.server({ httpServer: server });
    const script = ` new WebSocket('ws://' + window.location.host, 'elm-spa').onmessage = function () { window.location.reload() } `;
    ws.on('request', (req) => {
        const conn = req.accept('elm-spa', req.origin);
        connections[req.remoteAddress] = conn;
        conn.on('close', () => delete connections[conn.remoteAddress]);
    });
    // Send reload if any files change
    chokidar_1.default.watch(config.base, { ignoreInitial: true })
        .on('all', () => Object.values(connections).forEach(conn => conn.sendUTF('reload')));
    // Start server
    server.listen(config.port, () => resolve(`Ready at ${terminal_1.colors.cyan}http://localhost:${config.port}${terminal_1.reset}`));
    server.on('error', _ => {
        reject(`Unable to start server... is port ${config.port} in use?`);
    });
});
exports.default = {
    run: async () => {
        const output = await watch_1.watch();
        return start().then(serverOutput => [serverOutput, output]);
    }
};
